import { EditorView } from "@codemirror/view";
import { EditorState, TransactionSpec, ChangeDesc } from "@codemirror/state";
export interface Completion {
    label: string;
    displayLabel?: string;
    detail?: string;
    info?: string | ((completion: Completion) => CompletionInfo | Promise<CompletionInfo>);
    apply?: string | ((view: EditorView, completion: Completion, from: number, to: number) => void);
    type?: string;
    commitCharacters?: readonly string[];
    boost?: number;
    section?: string | CompletionSection;
}
export declare type CompletionInfo = Node | null | {
    dom: Node;
    destroy?(): void;
};
export interface CompletionSection {
    name: string;
    header?: (section: CompletionSection) => HTMLElement;
    rank?: number;
}
export declare class CompletionContext {
    readonly state: EditorState;
    readonly pos: number;
    readonly explicit: boolean;
    abortListeners: (() => void)[] | null;
    constructor(state: EditorState, pos: number, explicit: boolean);
    tokenBefore(types: readonly string[]): {
        from: number;
        to: number;
        text: string;
        type: import("@lezer/common").NodeType;
    };
    matchBefore(expr: RegExp): {
        from: number;
        to: number;
        text: string;
    };
    get aborted(): boolean;
    addEventListener(type: "abort", listener: () => void): void;
}
export declare function completeFromList(list: readonly (string | Completion)[]): CompletionSource;
export declare function ifIn(nodes: readonly string[], source: CompletionSource): CompletionSource;
export declare function ifNotIn(nodes: readonly string[], source: CompletionSource): CompletionSource;
export declare type CompletionSource = (context: CompletionContext) => CompletionResult | null | Promise<CompletionResult | null>;
export interface CompletionResult {
    from: number;
    to?: number;
    options: readonly Completion[];
    validFor?: RegExp | ((text: string, from: number, to: number, state: EditorState) => boolean);
    filter?: boolean;
    getMatch?: (completion: Completion, matched?: readonly number[]) => readonly number[];
    update?: (current: CompletionResult, from: number, to: number, context: CompletionContext) => CompletionResult | null;
    map?: (current: CompletionResult, changes: ChangeDesc) => CompletionResult | null;
    commitCharacters?: readonly string[];
}
export declare class Option {
    readonly completion: Completion;
    readonly source: CompletionSource;
    readonly match: readonly number[];
    score: number;
    constructor(completion: Completion, source: CompletionSource, match: readonly number[], score: number);
}
export declare function cur(state: EditorState): number;
export declare function ensureAnchor(expr: RegExp, start: boolean): RegExp;
export declare const pickedCompletion: import("@codemirror/state").AnnotationType<Completion>;
export declare function insertCompletionText(state: EditorState, text: string, from: number, to: number): TransactionSpec;
export declare function asSource(source: CompletionSource | readonly (string | Completion)[]): CompletionSource;
export declare const startCompletionEffect: import("@codemirror/state").StateEffectType<boolean>;
export declare const closeCompletionEffect: import("@codemirror/state").StateEffectType<null>;
