import { EditorView, Tooltip } from "@codemirror/view";
import { Transaction, StateField, EditorState, ChangeDesc } from "@codemirror/state";
import { Option, CompletionSource, CompletionResult } from "./completion";
import { CompletionConfig } from "./config";
declare class CompletionDialog {
    readonly options: readonly Option[];
    readonly attrs: {
        [name: string]: string;
    };
    readonly tooltip: Tooltip;
    readonly timestamp: number;
    readonly selected: number;
    readonly disabled: boolean;
    constructor(options: readonly Option[], attrs: {
        [name: string]: string;
    }, tooltip: Tooltip, timestamp: number, selected: number, disabled: boolean);
    setSelected(selected: number, id: string): CompletionDialog;
    static build(active: readonly ActiveSource[], state: EditorState, id: string, prev: CompletionDialog | null, conf: Required<CompletionConfig>): CompletionDialog | null;
    map(changes: ChangeDesc): CompletionDialog;
}
export declare class CompletionState {
    readonly active: readonly ActiveSource[];
    readonly id: string;
    readonly open: CompletionDialog | null;
    constructor(active: readonly ActiveSource[], id: string, open: CompletionDialog | null);
    static start(): CompletionState;
    update(tr: Transaction): CompletionState;
    get tooltip(): Tooltip | null;
    get attrs(): {
        [name: string]: string;
    };
}
export declare const enum State {
    Inactive = 0,
    Pending = 1,
    Result = 2
}
export declare function getUserEvent(tr: Transaction, conf: Required<CompletionConfig>): "input" | "delete" | null;
export declare class ActiveSource {
    readonly source: CompletionSource;
    readonly state: State;
    readonly explicitPos: number;
    constructor(source: CompletionSource, state: State, explicitPos?: number);
    hasResult(): this is ActiveResult;
    update(tr: Transaction, conf: Required<CompletionConfig>): ActiveSource;
    handleUserEvent(tr: Transaction, type: "input" | "delete", conf: Required<CompletionConfig>): ActiveSource;
    handleChange(tr: Transaction): ActiveSource;
    map(changes: ChangeDesc): ActiveSource;
}
export declare class ActiveResult extends ActiveSource {
    readonly result: CompletionResult;
    readonly from: number;
    readonly to: number;
    constructor(source: CompletionSource, explicitPos: number, result: CompletionResult, from: number, to: number);
    hasResult(): this is ActiveResult;
    handleUserEvent(tr: Transaction, type: "input" | "delete", conf: Required<CompletionConfig>): ActiveSource;
    handleChange(tr: Transaction): ActiveSource;
    map(mapping: ChangeDesc): ActiveSource | ActiveResult;
}
export declare const setActiveEffect: import("@codemirror/state").StateEffectType<readonly ActiveSource[]>;
export declare const setSelectedEffect: import("@codemirror/state").StateEffectType<number>;
export declare const completionState: StateField<CompletionState>;
export declare function applyCompletion(view: EditorView, option: Option): boolean;
export {};
